package com.brilliance.isc.funds.common.component;

import com.brilliance.isc.bo.Cbb;
import com.brilliance.isc.bo.Fxd;
import com.brilliance.isc.bo.Fxt;
import com.brilliance.isc.bo.model.Cpacbs;
import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.component.PtsptaToolComponent;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.exception.CommonServiceException;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.util.StringUtils;
import com.brilliance.isc.common.util.Utils;
import com.brilliance.isc.common.vo.PtsptaVo;
import com.brilliance.isc.mda.dao.FxdMapper;
import com.brilliance.isc.mda.dao.FxtMapper;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import com.brilliance.mda.runtime.mda.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

import static com.brilliance.mda.runtime.mda.Constants.NULLSTR;

@Component
public class FxToolComponent {

    @Autowired
    private FxdMapper fxdMapper;

    @Autowired
    private FxtMapper fxtMapper;

    @Autowired
    private PtsptaToolComponent ptsptaToolComponent;

    @Autowired
    private CbsmodService cbsmodService;

    @Autowired
    private SysmodService sysmodService;

    @Autowired
    private TrnmodService trnmodService;


    public Fxdgrp getFxdgrpByInr(String inr) {
        Fxd rec = new Fxd();
        rec.setInr(inr);
        rec = fxdMapper.selectByPrimaryKey(rec);
        if (Objects.isNull(rec)) {
            throw new CommonServiceException("主键" + inr + "找不到FXD数据");
        }
        Fxdgrp fxdgrp = new Fxdgrp();
        fxdgrp.reset();
        fxdgrp.setRec(rec);
        Fxt blk = new Fxt();
        blk.setInr(inr);
        blk = fxtMapper.selectByPrimaryKey(blk);
        fxdgrp.setBlk(blk);
        //获取cbs金额信息
        Date date = Dates.add(Utils.getMaxDate(), -1);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getMax(), "CPD", inr, "MAXSUM", "AMT1", date);
        cbsmodService.cbsGetBalance(fxdgrp.getCbs().getNom1(), "CPD", inr, "NOMSUM", "AMT1", date);
        //获取ptspta信息
        storePtsToGrp(fxdgrp, inr, "FXD");
        return fxdgrp;
    }


    public void storePtsToGrp(Object grpObject, String objInr, String objType) {
        //找ptspta数据
        Map<String, PtsptaVo> maps = ptsptaToolComponent.getPtsptaList(objInr, objType);
        if (maps.size() > 0) {
            maps.keySet().forEach(key -> {
                PtsptaVo ptsptaVo = maps.get(key);
                if (key.equalsIgnoreCase("INT")) {
                    key = "tmpint";
                }
                Utils.setFieldValue(grpObject, key.toLowerCase(), ptsptaVo);
            });
        }
    }

    public void collectCbsMap(TransactionServiceSet set, String prefix, Fxacbs cbs, String... cbsNames) {
        Map<String, Cbb> cbsMap = set.getCbsMap();
        if (cbsMap == null) {
            cbsMap = new HashMap<>();
        }
        for (String name : cbsNames) {
            Cbb cbb = Utils.getFieldValue(cbs, name);
            if (cbb != null) {
                cbsMap.put(prefix.toUpperCase() + name.toUpperCase(), cbb);
            }
        }
        set.setCbsMap(cbsMap);
    }


    public void collectPtsList(TransactionServiceSet set, Fxdgrp fxdgrp, String... ptsNames) {
        if (ptsNames == null) {
            //找所有的ptspta对象
            Field[] fields = fxdgrp.getClass().getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                if (PtsptaVo.class.equals(field.getType())) {
                    PtsptaVo ptspta = Utils.getFieldValue(fxdgrp, field.getName());
                    if (ptspta != null) {
                        set.addPtspta(transferFrom(ptspta));
                    }
                }
            }
        } else {
            for (String name : ptsNames) {
                PtsptaVo ptspta = Utils.getFieldValue(fxdgrp, name);
                if (ptspta != null && ptspta.getPts() != null && StringUtils.isNotEmpty(ptspta.getPts().getRol())) {
                    set.addPtspta(transferFrom(ptspta));
                }
            }
        }
    }


    /**
     * 根据STL结构的ptspta转化成抽象组件的ptspta，结构不一样
     *
     * @param ptsptas
     * @return List<com.brilliance.common.vo.PtsptaVo>
     */
    protected PtsptaVo transferFrom(PtsptaVo ptsptas) {
        PtsptaVo ptspta = ptsptas;
        ptspta.setRol(ptsptas.getPts().getRol());
        ptspta.setPtsinr(ptsptas.getPts().getInr());
        ptspta.setNam(ptsptas.getPts().getNam());
        ptspta.setPtyinr(ptsptas.getPts().getPtyinr());
        ptspta.setPtainr(ptsptas.getPts().getPtainr());
        ptspta.setExtkey(ptsptas.getPts().getExtkey());
        ptspta.setDftdsp(ptsptas.getPts().getDftdsp());
        ptspta.setDftcur(ptsptas.getPts().getDftcur());
        ptspta.setDftact(ptsptas.getPts().getDftact());
        ptspta.setDftfeecur(ptsptas.getPts().getDftfeecur());
        ptspta.setDftactptainr(ptsptas.getPts().getDftactptainr());
        ptspta.setDftselrol(ptsptas.getPts().getDftsetrol());
        ptspta.setGlggrpflg(ptsptas.getPts().getGlggrpflg());
        ptspta.setRef(ptsptas.getPts().getRef());
        ptspta.setPtytyp(ptsptas.getPtytyp());
        ptspta.setPtyextkey(ptsptas.getPtyextkey());
        ptspta.setAdrblk(ptsptas.getPts().getAdrblk());
        return ptspta;
    }


    /**
     * source:fxtmod.0000.script
     */
    public void getref(Fxdgrp fxdgrp) {
        //! Set reference number, if not already set (usually called in transaction`s MTABUT\\SAV Event)
        String tbl = NULLSTR;
        int len = 0;
        switch (fxdgrp.getRec().getFxtyp()) {
            case "LS":
            case "LB":
                tbl = "FX";
                len = 4;
                break;
            case "XS":
            case "XB":
                tbl = "DX";
                len = 4;
                break;
            case "US":
                tbl = "S" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
                break;
            case "UB":
                tbl = "B" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
                break;
            default:
                tbl = "J" + Strings.mid(fxdgrp.getCbs().getMax().getCur(), 1, 1);
                len = 4;
        }
        if (MdaUtils.isEmpty(fxdgrp.getRec().getOwnref())) {

            fxdgrp.getRec().setOwnref(sysmodService.getRef(tbl, len));
        }

    }

    /**
     * source:fxtmod.0001.script
     */
    public void sav(Fxdgrp fxdgrp, BigDecimal cshpct) {
        // saves contract including texts. CBS resp. of transaction
        String frm = SettleContext.getTransName();
        switch (frm) {
            case "FXTSUS":
            case "FXTSUB":
            case "FXTSST":
            case "FXTSCN":
                //Datas.getContentNumeric(this, "\\LIAALL\\LIACCV\\CSHPCT")
                fxdgrp.getRec().setCshpct(cshpct);
                break;
        }
        trnmodService.trnStoreContract(fxdgrp);
    }

    /**
     * source:fxtmod.0002.script
     */
    public void setObjNam(String objref, String nam) {
        //Setting OBJNAM. If the setting of OBJREF is not standard, here you can change it.
//        ctx.absGet(Sysmod.class).setObjnam(NULLSTR);
//        ctx.absGet(Sysmod.class).setObjnam(objref + "/" + nam);

    }

}
