package com.brilliance.isc.funds.fxtlop.service.impl;

import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.liaall.service.LiaallService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.funds.bo.funds.FxtlopStoreBo;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtlop.check.FxtlopValidator;
import com.brilliance.isc.funds.fxtlop.service.FxtlopService;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Map;

/**
 * @program: isc-funds
 * @ClassName: FxtlopTransactionServiceImpl
 * @description: 结售汇平盘登记 Fxtlop 业务处理
 * @author: wangweidong
 * @create: 2024-10-08 16:35
 */
@Service("fxtlop.transaction")
public class FxtlopTransactionServiceImpl extends AbstractTransactionService<FxtlopStoreBo> {

    @Resource
    private FxtlopValidator fxtlopValidator;
    @Autowired
    private FxtlopService fxtlopService;
    @Resource
    private FxToolComponent fxToolComponent;
    @Resource
    private TrnmodService trnmodService;
    @Resource
    private TransactionHelpService transactionHelpService;
    @Resource
    private CbsmodService cbsmodService;
    @Resource
    private LiaallService liaallService;
    @Resource
    private SettleContext settleContext;



    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }


    @Override
    protected void txnSave(FxtlopStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    @Override
    protected Map<String, String> txnCheck(FxtlopStoreBo transactionVo) {
        return fxtlopValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtlopStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtlopStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtlopStoreBo init(FxtlopStoreBo fxtlopStoreBo) {
        fxtlopStoreBo.reset();
        fxtlopService.initFxtlop(fxtlopStoreBo);
        fxtlopService.defaultFxtlop(fxtlopStoreBo);
        return fxtlopStoreBo;
    }

    @Override
    public void beforeSave(FxtlopStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtlopStoreBo transactionVo) {
    }

    public void ruleMtabutSavN500(FxtlopStoreBo transactionVo){
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        fxToolComponent.getref(transactionVo.getFxdgrp());
        // 1. Create (rsp. update) Contract (incl. Texts)
        fxToolComponent.sav(transactionVo.getFxdgrp(), BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract ( trnmodVo);
        cbsSav(transactionVo, trnmodVo);
        // save documents
    }


    public void cbsSav(FxtlopStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN",trnmodVo.getTrn().getInr());
        //#cgl
        //平盘登记后产生4条CBB,MAX,NOM,OPN,OPN2
        //后续两个交易分别消OPN和OPN2
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fxtlop","CT000000"),transactionVo.getFxdgrp(),trninr);
        trnmodService.cbsTrnStore("NOMAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getNom1().getCur(),transactionVo.getFxdgrp().getCbs().getNom1().getAmt(),MdaUtils.getI18NString("fxtlop","CT000001"),transactionVo.getFxdgrp(),trninr);
        /**
         if FXDGRP\\REC\\FXTYP.is ( "LB" ) or FXDGRP\\REC\\FXTYP.is ( "LS" ) then

         **/
        /**
         if FXDGRP\\REC\\FXTYP.is ( "LB" ) then

         **/
        if( MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LB")== 0 || MdaUtils.compareTo(transactionVo.getFxdgrp().getRec().getFxtyp(),"LS")== 0 )
        {
            trnmodService.cbsTrnStore("OPNAMT","AMT1",Dates.today(),transactionVo.getFxdgrp().getCbs().getMax().getCur(),transactionVo.getFxdgrp().getCbs().getMax().getAmt(),MdaUtils.getI18NString("fxtlop","CT000002"),transactionVo.getFxdgrp(),trninr);
        }
        liaallService.storeAll(transactionHelpService.getTransactionStoreSet().getLiaall());
        //Platform.cbsTrnCommit();
    }
}
