package com.brilliance.isc.funds.fxtssb.service;

import com.brilliance.isc.bo.model.Fxacbs;
import com.brilliance.isc.bo.model.Fxdgrp;
import com.brilliance.isc.common.cbsmod.service.CbsmodService;
import com.brilliance.isc.common.cbsmod.service.XrtmodService;
import com.brilliance.isc.common.contants.CommonContants;
import com.brilliance.isc.common.context.SettleContext;
import com.brilliance.isc.common.sysmod.SysmodService;
import com.brilliance.isc.common.transaction.AbstractTransactionService;
import com.brilliance.isc.common.transaction.help.TransactionHelpService;
import com.brilliance.isc.common.transaction.help.TransactionServiceSet;
import com.brilliance.isc.common.trnmod.service.TrnmodService;
import com.brilliance.isc.common.trnmod.vo.TrnmodVo;
import com.brilliance.isc.common.util.I18nUtil;
import com.brilliance.isc.common.vo.BizAmdInfoVo;
import com.brilliance.isc.common.vo.BizInfoVo;
import com.brilliance.isc.funds.bo.funds.FxtssbStoreBo;
import com.brilliance.isc.funds.common.component.FxToolComponent;
import com.brilliance.isc.funds.fxtssb.check.FxtssbValidator;
import com.brilliance.isc.vo.funds.FxdBizInfoVo;
import com.brilliance.mda.runtime.mda.impl.Argument;
import com.brilliance.mda.runtime.mda.util.Dates;
import com.brilliance.mda.runtime.mda.util.MdaUtils;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Map;

/**
 * @program: isc-funds
 * @ClassName: FxtssbTransactionServiceImpl
 * @description: 即期结汇 Fxtssb 业务处理
 * @author: huangshunlin
 * @create: 2024-10-08 15:35
 */
@Service("fxtssb.transaction")
public class FxtssbTransactionServiceImpl extends AbstractTransactionService<FxtssbStoreBo> {

    @Resource
    private FxtssbValidator fxtssbValidator;

    @Resource
    private XrtmodService xrtmodService;

    @Resource
    private SysmodService sysmodService;

    @Resource
    private FxToolComponent fxToolComponent;


    @Resource
    private TransactionHelpService transactionHelpService;


    @Resource
    private SettleContext settleContext;

    @Resource
    private CbsmodService cbsmodService;

    @Resource
    private TrnmodService trnmodService;

    @PostConstruct
    public void regisertService() {
        serviceRegisterMaps.put(CommonContants.SETMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDOC_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.GLEMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNDIA_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.TRNMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.UMDMOD_SERVICE, "N");
        serviceRegisterMaps.put(CommonContants.LIMMOD_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_RMB_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_SZSB_BOPREM_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.BOPSET_CFABOP_SERVICE, "Y");
        serviceRegisterMaps.put(CommonContants.FANXIQIAN_SERVICE, "Y");
    }


    @Override
    protected void txnSave(FxtssbStoreBo transactionVo) {
        ruleMtabutSavN500(transactionVo);
    }

    public void ruleMtabutSavN500(FxtssbStoreBo bo) {
        TrnmodVo trnmodVo = transactionHelpService.getTransactionStoreSet().getTrnmodVo();

        Fxdgrp fxdgrp = bo.getFxdgrp();
        fxToolComponent.getref(fxdgrp);
        // 1. Create (rsp. update) Contract (incl. Texts)
        Argument<String> pathBox = new Argument<String>("", fxdgrp.getRec().getAccmng());
        Argument<String> pathBox1 = new Argument<String>("", fxdgrp.getRec().getHdbch());
        sysmodService.getPtyAccMng(fxdgrp.getApl().getPta().getPtyinr(), "", "", "", pathBox, pathBox1);

        fxdgrp.getRec().setAccmng(pathBox.value);
        fxdgrp.getRec().setHdbch(pathBox1.value);
        fxToolComponent.sav(fxdgrp, BigDecimal.ZERO);
        // 2. Build Transaction from FTDGRP
        trnmodService.trnSetFromContract(trnmodVo);

        cbsSav(bo, trnmodVo);
        // save documents

    }

    @Override
    protected Map<String, String> txnCheck(FxtssbStoreBo transactionVo) {
        return fxtssbValidator.validate(transactionVo);
    }

    @Override
    protected BizInfoVo buildBizInfo(FxtssbStoreBo transactionVo) {
        FxdBizInfoVo bizInfoVo = new FxdBizInfoVo();
        bizInfoVo.setObjtyp("FXD");
        return bizInfoVo;
    }

    @Override
    protected BizAmdInfoVo buildBizAmdInfo(FxtssbStoreBo transactionVo) {
        return null;
    }

    @Override
    public FxtssbStoreBo init(FxtssbStoreBo fxtssbStoreBo) {
        fxtssbStoreBo.reset();
        init950(fxtssbStoreBo);
        init1000(fxtssbStoreBo);
        return fxtssbStoreBo;
    }

    /**
     * source:fxtssb.0001.script
     */
    public void init1000(FxtssbStoreBo bo) {

        bo.getFxdgrp().getCbs().getNom1().setCur(xrtmodService.sysiso());
        // MAXAMT = 200000.00
//        Systems.setValues(this.getFxdgrp().getRec(), "fxtyp", "SB");
        bo.getFxdgrp().getRec().setFxtyp("SB");
        String extkey = SettleContext.getUserSession().getUsr().getExtkey();
        bo.getFxdgrp().getRec().setUsr(extkey);

    }

    private void init950(FxtssbStoreBo fxtssbStoreBo) {
    }

    @Override
    public void beforeSave(FxtssbStoreBo transactionVo) {
        transactionVo.reset();
        Fxdgrp fxdgrp = transactionVo.getFxdgrp();
        TransactionServiceSet transactionServiceSet = transactionHelpService.getTransactionStoreSet();
        transactionServiceSet.setRec(fxdgrp.getRec());
        transactionServiceSet.setBlk(fxdgrp.getBlk());
        transactionServiceSet.setTransactionVo(transactionVo);
        fxToolComponent.collectCbsMap(transactionServiceSet, "", fxdgrp.getCbs(), "max", "nom1", "opn1", "opn2");
        fxToolComponent.collectPtsList(transactionServiceSet, fxdgrp, null);
        settleContext.loadSettleSession(transactionVo.getTransName(), transactionVo.getUserId(), buildBizInfo(transactionVo), transactionServiceSet.getPtsList());
    }

    @Override
    protected void updateBizInfoVo(BizInfoVo bizInfoVo, FxtssbStoreBo transactionVo) {
        bizInfoVo.setObjinr(transactionVo.getFxdgrp().getRec().getInr());
    }


    /**
     * source:fxtssb.0003.script
     */
    public void cbsSav(FxtssbStoreBo transactionVo, TrnmodVo trnmodVo) {
        cbsmodService.cbsTrnSetNew("TRN", trnmodVo.getTrn().getInr());
        Fxacbs cbs = transactionVo.getFxdgrp().getCbs();
        String trninr = transactionHelpService.getTrninr();
        trnmodService.cbsTrnStore("MAXAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
        if (!MdaUtils.isEmpty(cbs.getNom1().getAmt())) {
            trnmodService.cbsTrnStore("NOMAMT", "AMT1", Dates.today(), cbs.getNom1().getCur(), cbs.getNom1().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
            trnmodService.cbsTrnStore("UTLAMT", "AMT1", Dates.today(), cbs.getMax().getCur(), cbs.getMax().getAmt(), I18nUtil.getMessage("funds_fxtssb_000000"), transactionVo.getFxdgrp(), trninr);
        }
    }
}
